#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

import sympy as sp
from sympy.physics.quantum import TensorProduct
import cupy as cp
# try:
#  import cupy as cp
# except Exception:
#  raise RuntimeError('CuPy is not available!')

from logicqubit.utils import *


class Hilbert():
    __cuda = True
    __first_left = True

    def ket(self, value, d=2):
        result = Matrix([[Utils.onehot(i, value)] for i in range(d)], Hilbert.__cuda)
        return result

    def bra(self, value, d=2):
        result = Matrix([Utils.onehot(i, value) for i in range(d)], Hilbert.__cuda)
        return result

    def getAdjoint(self, psi):
        result = psi.adjoint()
        return result

    def product(self, Operator, psi):
        result = Operator * psi
        return result

    def kronProduct(self, list):  # produto de Kronecker
        A = list[0]  # atua no qubit 1 que é o mais a esquerda
        for M in list[1:]:
            A = A.kron(M)
        return A

    def setNumberOfQubits(self, number):
        Hilbert.__number_of_qubits = number

    def getNumberOfQubits(self):
        return Hilbert.__number_of_qubits

    def setCuda(self, cuda):
        Hilbert.__cuda = cuda

    def getCuda(self):
        return Hilbert.__cuda

    def setFirstLeft(self, value):
        Hilbert.__first_left = value

    def isFirstLeft(self):
        return Hilbert.__first_left


class Matrix:

    def __init__(self, matrix, cuda=True):
        self.__matrix = matrix
        self.__cuda = cuda
        if isinstance(matrix, list):
            if self.__cuda:
                self.__matrix = cp.array(matrix)
            else:
                self.__matrix = sp.Matrix(matrix)
        else:
            self.__matrix = matrix

    def __add__(self, other):
        result = self.__matrix + other.get()
        return Matrix(result, self.__cuda)

    def __sub__(self, other):
        result = self.__matrix - other.get()
        return Matrix(result, self.__cuda)

    def __mul__(self, other):
        if isinstance(other, Matrix):
            other = other.get()
            if self.__cuda:
                result = cp.dot(self.__matrix, other)
            else:
                result = self.__matrix * other
        else:
            result = self.__matrix * other
        return Matrix(result, self.__cuda)

    def __truediv__(self, other):
        result = self.__matrix * (1./other)
        return Matrix(result, self.__cuda)

    def __eq__(self, other):
        return self.__matrix == other.get()

    def __str__(self):
        return str(self.__matrix)

    def kron(self, other):  # Kronecker product
        if self.__cuda:
            result = cp.kron(self.__matrix, other.get())
        else:
            result = TensorProduct(self.__matrix, other.get())
        return Matrix(result, self.__cuda)

    def get(self):
        return self.__matrix

    def trace(self):
        result = self.__matrix.trace()
        return Matrix(result, self.__cuda)

    def adjoint(self):
        if self.__cuda:
            result = self.__matrix.transpose().conj()
        else:
            result = self.__matrix.transpose().conjugate()
        return Matrix(result, self.__cuda)



#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

import sympy as sp
import numpy as np
from sympy.physics.quantum import TensorProduct
from cmath import *
import random

from mpl_toolkits.mplot3d import Axes3D
import matplotlib.pyplot as plt
from matplotlib import cm

from logicqubit.qubits import *
from logicqubit.gates import *
from logicqubit.circuit import *
from logicqubit.zhegalkin import *
from logicqubit.oracle import *
from logicqubit.utils import *

class LogicQuBit(Qubits, Gates, Circuit):

    def __init__(self, number_of_qubits = 3, **kwargs):
        self.__cuda = kwargs.get('cuda', True)
        self.__symbolic = kwargs.get('symbolic', False)
        self.__first_left = kwargs.get('first_left', True)  # o qubit 1 é o mais a esquerda
        if self.__symbolic:
            self.__cuda = False
        super().setCuda(self.__cuda)
        super().setFirstLeft(self.__first_left)
        super().setNumberOfQubits(number_of_qubits)
        Qubits.__init__(self)
        Gates.__init__(self, number_of_qubits)
        Circuit.__init__(self)

    def X(self, target):
        self.addOp("X", self.qubitsToList([target]))
        operator = super().X(target)
        self.setOperation(operator)

    def Y(self, target):
        self.addOp("Y", self.qubitsToList([target]))
        operator = super().Y(target)
        self.setOperation(operator)

    def Z(self, target):
        self.addOp("Z", self.qubitsToList([target]))
        operator = super().Z(target)
        self.setOperation(operator)

    def V(self, target):
        self.addOp("V", self.qubitsToList([target]))
        operator = super().V(target)
        self.setOperation(operator)

    def S(self, target):
        self.addOp("S", self.qubitsToList([target]))
        operator = super().S(target)
        self.setOperation(operator)

    def T(self, target):
        self.addOp("T", self.qubitsToList([target]))
        operator = super().T(target)
        self.setOperation(operator)

    def H(self, target):
        self.addOp("H", self.qubitsToList([target]))
        operator = super().H(target)
        self.setOperation(operator)

    def U(self, target, *argv):
        self.addOp("U", self.qubitsToList([target, argv]))
        operator = super().U(target, argv)
        self.setOperation(operator)

    def U3(self, target, theta, phi, _lambda):
        self.addOp("U3", self.qubitsToList([target, theta, phi, _lambda]))
        operator = super().U3(target, theta, phi, _lambda)
        self.setOperation(operator)

    def U2(self, target, phi, _lambda):
        self.addOp("U2", self.qubitsToList([target, phi, _lambda]))
        operator = super().U2(target, phi, _lambda)
        self.setOperation(operator)

    def U1(self, target, _lambda):
        self.addOp("U1", self.qubitsToList([target, _lambda]))
        operator = super().U1(target, _lambda)
        self.setOperation(operator)

    def RX(self, target, theta):
        self.addOp("RX", self.qubitsToList([target, theta]))
        operator = super().RX(target, theta)
        self.setOperation(operator)

    def RY(self, target, theta):
        self.addOp("RY", self.qubitsToList([target, theta]))
        operator = super().RY(target, theta)
        self.setOperation(operator)

    def RZ(self, target, phi):
        self.addOp("RZ", self.qubitsToList([target, phi]))
        operator = super().RZ(target, phi)
        self.setOperation(operator)

    def CX(self, control, target):
        self.addOp("CX", self.qubitsToList([control, target]))
        operator = super().CX(control, target)
        self.setOperation(operator)

    def CNOT(self, control, target):
        self.CX(control, target)

    def CY(self, control, target):
        self.addOp("CY", self.qubitsToList([control, target]))
        operator = super().CY(control, target)
        self.setOperation(operator)

    def CZ(self, control, target):
        self.addOp("CZ", self.qubitsToList([control, target]))
        operator = super().CZ(control, target)
        self.setOperation(operator)

    def CRX(self, control, target, theta):
        self.addOp("CRX", self.qubitsToList([control, target, theta]))
        operator = super().CRX(control, target, theta)
        self.setOperation(operator)

    def CRY(self, control, target, theta):
        self.addOp("CRY", self.qubitsToList([control, target, theta]))
        operator = super().CRY(control, target, theta)
        self.setOperation(operator)

    def CRZ(self, control, target, phi):
        self.addOp("CRZ", self.qubitsToList([control, target, phi]))
        operator = super().CRZ(control, target, phi)
        self.setOperation(operator)

    def CU(self, control, target, *argv):
        self.addOp("CU", self.qubitsToList([control, target]))
        operator = super().CU(control, target, argv)
        self.setOperation(operator)

    def CU3(self, control, target, theta, phi, _lambda):
        self.addOp("CU3", self.qubitsToList([control, target, theta, phi, _lambda]))
        operator = super().CU3(control, target, theta, phi, _lambda)
        self.setOperation(operator)

    def CU2(self, control, target, phi, _lambda):
        self.addOp("CU2", self.qubitsToList([control, target, phi, _lambda]))
        operator = super().CU2(control, target, phi, _lambda)
        self.setOperation(operator)

    def CU1(self, control, target, _lambda):
        self.addOp("CU1", self.qubitsToList([control, target, _lambda]))
        operator = super().CU1(control, target, _lambda)
        self.setOperation(operator)

    def CCX(self, control1, control2, target):
        self.addOp("CCX", self.qubitsToList([control1, control2, target]))
        operator = super().CCX(control1, control2, target)
        self.setOperation(operator)

    def Toffoli(self, control1, control2, target):
        self.CCX(control1, control2, target)

    def addOracle(self, oracle):
        targets, input_qubits, truth_table = oracle.get()
        poly = Zhegalkin_Poly()
        for bits in truth_table:
            poly.addTable(bits)
        result = poly.Compute()
        for target, p in enumerate(result):
            for i, value in reversed(list(enumerate(p))):
                if(value==1):
                    blist = [int(i, base=2) for i in bin(i)[2:].zfill(len(input_qubits))]
                    if(bin(i).count("1") == 2):
                        try:
                            q1 = blist.index(1)
                            q2 = blist[q1+1:].index(1)+q1+1
                            self.CCX(input_qubits[q1], input_qubits[q2], targets[target])
                        except:
                            print('fail')
                    elif(bin(i).count("1")==1):
                        try:
                            q = blist.index(1)
                            self.CX(input_qubits[q], targets[target])
                        except:
                            print('fail')
                    elif(i==0):
                        self.X(targets[target])

    def DensityMatrix(self):
        density_m = self.getPsi() * self.getPsiAdjoint()
        return density_m

    def Pure(self):
        density_m = self.DensityMatrix()
        pure = (density_m*density_m).trace()
        return pure

    def get_shot(self, measured, shots):
        max_set = shots*100
        if(self.__symbolic):
            list_all = [int(value * max_set) * [i] for i, value in enumerate(measured)]
        else:
            list_all = [int(value.real * max_set) * [i] for i, value in enumerate(measured)]
        list_all_values = []
        for list_values in list_all:
            list_all_values += list_values
        values = [random.choice(list_all_values) for i in range(shots)]
        return values

    def Measure_One(self, target, shots=1):
        self.addOp("Measure", self.qubitsToList([target]))
        density_m = self.DensityMatrix()
        list = self.getOrdListSimpleGate(target, super().P0())
        P0 = self.kronProduct(list)
        list = self.getOrdListSimpleGate(target, super().P1())
        P1 = self.kronProduct(list)
        measure_0 = (density_m*P0).trace().get()
        measure_1 = (density_m*P1).trace().get()
        value = self.get_shot([measure_0, measure_1], shots)
        if(value[0] == 0):
            new_state = self.product(P0, self.getPsi())/sqrt(measure_0)
        else:
            new_state = self.product(P1, self.getPsi())/sqrt(measure_1)
        self.setPsi(new_state)
        return value

    def Measure(self, target, fisrt_msb = False):  # ex: medir 3 qubits de 5: 2,1,4 do estado "010" -> M010 = |1><1| x |0><0| x 1 x |0><0| x 1
        target = self.qubitsToList(target)
        if(fisrt_msb):  # se fisrt_msb=True -> o primeiro da lista será o mais significativo
            target.reverse()
        self.setMeasuredQubits(target)
        self.addOp("Measure", target)
        density_m = self.DensityMatrix()
        size_p = len(target)  # número de qubits a ser medidos
        size = 2 ** size_p  # número de estados possíveis
        result = []
        for i in range(size):
            tlist = [self.ID() for tl in range(self.getNumberOfQubits())]
            blist = [i >> bl & 0x1 for bl in range(size_p)]  # bit list, bits de cada i
            for j, value in enumerate(target):
                if blist[j] == 0:  # mais significativo primeiro
                    tlist[value-1] = super().P0()
                else:
                    tlist[value-1] = super().P1()
            if (not self.__first_left):
                tlist.reverse()
            M = self.kronProduct(tlist)
            measure = (density_m * M).get().trace()  # valor esperado
            if(self.__cuda):
                measure = measure.get().item().real
            result.append(measure)
        self.setMeasuredValues(result)
        return result

    def Plot(self, big_endian=False):
        size_p = len(self.getMeasuredQubits())  # número de bits medidos
        if(size_p > 0):
            size = 2 ** size_p
            if(big_endian):
                names = ["|" + ''.join(list(reversed("{0:b}".format(i).zfill(size_p)))) + ">" for i in range(size)]
            else:
                names = ["|" + "{0:b}".format(i).zfill(size_p) + ">" for i in range(size)]
            values = self.getMeasuredValues()
            #plt.figure(figsize = (5, 3))
            plt.bar(names, values)
            plt.xticks(rotation=50)
            plt.suptitle('')
            plt.show()
        else:
            print("No qubit measured!")

    def PlotDensityMatrix(self, imaginary=False, decimal=False):
        size_p = self.getNumberOfQubits()  # número de qubits
        mRho = [[0]*2**size_p for i in range(2**size_p)]
        rho = self.DensityMatrix().get()
        for id1 in range(2**size_p):
            for id2 in range(2**size_p):
                if (self.__cuda):
                    value = rho[id1][id2]
                    if(not imaginary):
                        value = value.item().real
                    else:
                        value = value.item().imag
                else:
                    value = rho[id1,id2]
                    if(not imaginary):
                        value = sp.re(value)
                    else:
                        value = sp.im(value)
                mRho[id1][id2] = value

        result = np.array(mRho, dtype=np.float)
        fig = plt.figure(figsize=(5, 5), dpi=150)
        ax1 = fig.add_subplot(111, projection='3d')

        size = 2 ** size_p
        if(not decimal):
            labels = ["|" + "{0:b}".format(i).zfill(size_p) + ">" for i in range(size)]
        else:
            labels = [i for i in range(size)]
        xlabels = np.array(labels)
        xpos = np.arange(xlabels.shape[0])
        ylabels = np.array(labels)
        ypos = np.arange(ylabels.shape[0])

        xpos_mesh, ypos_mesh = np.meshgrid(xpos, ypos, copy=False)

        zpos = result
        zpos = zpos.ravel()

        dx = 0.5
        dy = 0.5
        dz = zpos

        ax1.w_xaxis.set_ticks(xpos + dx / 2.)
        ax1.w_xaxis.set_ticklabels(xlabels)
        ax1.w_yaxis.set_ticks(ypos + dy / 2.)
        ax1.w_yaxis.set_ticklabels(ylabels)

        values = np.linspace(0.2, 1., xpos_mesh.ravel().shape[0])
        colors = cm.rainbow(values)
        ax1.bar3d(xpos_mesh.ravel(), ypos_mesh.ravel(), dz * 0, dx, dy, dz, color=colors)
        plt.show()


#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

import sympy as sp
from sympy.physics.quantum import TensorProduct
from IPython.display import display, Math, Latex

from logicqubit.hilbert import *
from logicqubit.gates import *
from logicqubit.circuit import *
from logicqubit.utils import *

class Qubits(Hilbert):

    def __init__(self):
        Qubits.__number = 0
        Qubits.__used_qubits = []
        Qubits.__measured_qubits = []
        Qubits.__measured_values = []
        if self.getCuda():
            Qubits.__psi = self.kronProduct([self.ket(0) for i in range(self.getNumberOfQubits())])
        else:
            if self.isFirstLeft():  # o qubit 1 é o primeiro a esquerda
                a = sp.symbols([str(i) + "a" + str(i) + "_0" for i in range(1, self.getNumberOfQubits() + 1)])
                b = sp.symbols([str(i) + "b" + str(i) + "_1" for i in range(1, self.getNumberOfQubits() + 1)])
            else:
                a = sp.symbols([str(self.getNumberOfQubits()+1-i) + "a" + str(i) + "_0" for i in reversed(range(1, self.getNumberOfQubits() + 1))])
                b = sp.symbols([str(self.getNumberOfQubits()+1-i) + "b" + str(i) + "_1" for i in reversed(range(1, self.getNumberOfQubits() + 1))])
            Qubits.__psi = self.kronProduct([self.ket(0)*a[i]+self.ket(1)*b[i] for i in range(self.getNumberOfQubits())])

    def addQubit(self, id=None):
        if(len(Qubits.__used_qubits) < self.getNumberOfQubits()):
            if(id != None):
                if(not id in Qubits.__used_qubits):
                    Qubits.__used_qubits.append(id)
                else:
                    print("Qubit already used!")
            else:
                id = self.getLowestIdAvailable()
                Qubits.__used_qubits.append(id)
            return id
        else:
            print("All qubits have already been used!")
            return None

    def getLowestIdAvailable(self):
        all = list(range(1, self.getNumberOfQubits() + 1))
        for i in Qubits.__used_qubits:
            all.remove(i)
        return min(all)

    def getBiggestIdAvailable(self):
        all = list(range(1, self.getNumberOfQubits() + 1))
        for i in Qubits.__used_qubits:
            all.remove(i)
        return max(all)

    def getQubitsNumber(self):
        return self.getNumberOfQubits()

    def getUsedQubits(self):
        return len(Qubits.__used_qubits)

    def setMeasuredQubits(self, target):
        if(isinstance(target, list)):
            for id in target:
                Qubits.__measured_qubits.append(id)
        else:
            Qubits.__measured_qubits.append(target)

    def getMeasuredQubits(self):
        return Qubits.__measured_qubits

    def isMeasured(self, target):
        if (isinstance(target, list)):
            for id in target:
                if(id in Qubits.__measured_qubits):
                    print("Qubit "+str(id)+" alredy measured!")
                    return True
        else:
            if(target in Qubits.__measured_qubits):
                return True
        return False

    def setMeasuredValues(self, value):
        Qubits.__measured_values = value

    def getMeasuredValues(self):
        return Qubits.__measured_values

    def setPsi(self, psi):
        Qubits.__psi = psi

    def getPsi(self):
        return Qubits.__psi

    def getDictPsi(self):
        size_p = self.getQubitsNumber()
        size = 2 ** size_p
        labels = ["{0:b}".format(i).zfill(size_p) for i in range(size)]
        if not self.getCuda():
            value_l = [Utils.textSymbolfix(str(value), self.getQubitsNumber(), self.isFirstLeft()) for value in Qubits.__psi.get()]
            dictPsi = {label: value_l[i] for i, label in enumerate(labels)}
        else:
            dictPsi = {label: Qubits.__psi.get()[i].item() for i, label in enumerate(labels)}
        return dictPsi

    def getPsiAtAngles(self, degree=False):
        angles = []
        if self.getCuda():
            angles = cp.angle(Qubits.__psi.get())
            if(degree):
                angles = angles*180/pi
        return angles

    def getPsiAdjoint(self):
        result = Qubits.__psi.adjoint()
        return result

    def setOperation(self, operator):
        Qubits.__psi = self.product(operator, Qubits.__psi)
        Qubits.__last_operator = operator

    def qubitsToList(self, values):
        result = []
        for value in values:
            if (isinstance(value, Qubit)):
                result.append(value.getId())
            else:
                result.append(value)
        return result

    def setSymbolValuesForAll(self, a, b):
        if not self.getCuda():
            for i in range(1, self.getNumberOfQubits()+1):
                if (self.isFirstLeft()):
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+"a"+str(i)+"_0", a), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+"a"+str(i)+"_1", a), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+"b"+str(i)+"_0", b), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+"b"+str(i)+"_1", b), False)
                else:
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+"a"+str(i)+"_0", a), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+"a"+str(i)+"_1", a), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+"b"+str(i)+"_0", b), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+"b"+str(i)+"_1", b), False)
        else:
            print("This session is not symbolic!")

    def setSymbolValuesForListId(self, id, a, b):
        if not self.getCuda():
            list_id = self.qubitsToList(id)
            for i in list_id:
                if (self.isFirstLeft()):
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+"a"+str(i)+"_0", a), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+"a"+str(i)+"_1", a), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+"b"+str(i)+"_0", b), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+"b"+str(i)+"_1", b), False)
                else:
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+"a"+str(i)+"_0", a), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+"a"+str(i)+"_1", a), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+"b"+str(i)+"_0", b), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+"b"+str(i)+"_1", b), False)
        else:
            print("This session is not symbolic!")

    def setSymbolValue(self, id, symbol, value):
        if not self.getCuda():
            list_id = self.qubitsToList(id)
            for i in list_id:
                if self.isFirstLeft():
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+symbol+str(i)+"_0", value), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(i)+symbol+str(i)+"_1", value), False)
                else:
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+symbol+str(i)+"_0", value), False)
                    Qubits.__psi = Matrix(Qubits.__psi.get().subs(str(self.getNumberOfQubits()+1-i)+symbol+str(i)+"_1", value), False)
        else:
            print("This session is not symbolic!")

    def PrintState(self, simple = False):
        if self.getCuda():
            value = Utils.vec2tex(Qubits.__psi.get())
        else:
            value = Utils.texfix(Qubits.__psi.get(), self.getNumberOfQubits(), self.isFirstLeft())

        if(not simple):
            display(Math(value))
        else:
            print(value)

    def PrintLastOperator(self, tex = True):
        if(tex):
            value = sp.latex(Qubits.__last_operator.get())
            display(Math(value))
        else:
            print(Qubits.__last_operator.get())


class Qubit(Qubits, Gates, Circuit):
    def __init__(self, id = None):
        self.__id = self.addQubit(id)
        self.__name = "q"+str(self.__id)
        Gates.__init__(self, self.getQubitsNumber())

    def __eq__(self, other):
        return self.__id == other

    def setName(self, name):
        self.__name = name

    def getName(self):
        return self.__name

    def getId(self):
        return self.__id

    def setSymbolValues(self, a, b):
        self.setSymbolValuesForListId([self.__id], a, b)

    def X(self):
        self.addOp("X", [self.__id])
        operator = super().X(self.__id)
        self.setOperation(operator)

    def Y(self):
        self.addOp("Y", [self.__id])
        operator = super().Y(self.__id)
        self.setOperation(operator)

    def Z(self):
        self.addOp("Z", [self.__id])
        operator = super().Z(self.__id)
        self.setOperation(operator)

    def V(self, target):
        self.addOp("V", [self.__id])
        operator = super().V(self.__id)
        self.setOperation(operator)

    def S(self, target, adjoint = False):
        self.addOp("S", [self.__id])
        operator = super().S(self.__id, adjoint)
        self.setOperation(operator)

    def T(self, target):
        self.addOp("T", [self.__id])
        operator = super().T(self.__id)
        self.setOperation(operator)

    def H(self):
        self.addOp("H", [self.__id])
        operator = super().H(self.__id)
        self.setOperation(operator)

    def U(self, *argv):
        self.addOp("U", self.qubitsToList([self.__id, argv]))
        operator = super().U(self.__id, argv)
        self.setOperation(operator)

    def U3(self, theta, phi, _lambda):
        self.addOp("U3", self.qubitsToList([self.__id, theta, phi, _lambda]))
        operator = super().U3(self.__id, theta, phi, _lambda)
        self.setOperation(operator)

    def U2(self, phi, _lambda):
        self.addOp("U2", self.qubitsToList([self.__id, phi, _lambda]))
        operator = super().U2(self.__id, phi, _lambda)
        self.setOperation(operator)

    def U1(self, _lambda):
        self.addOp("U1", self.qubitsToList([self.__id, _lambda]))
        operator = super().U1(self.__id, _lambda)
        self.setOperation(operator)

    def RX(self, theta):
        self.addOp("RX", self.qubitsToList([self.__id, theta]))
        operator = super().RX(self.__id, theta)
        self.setOperation(operator)

    def RY(self, theta):
        self.addOp("RY", self.qubitsToList([self.__id, theta]))
        operator = super().RY(self.__id, theta)
        self.setOperation(operator)

    def RZ(self, phi):
        self.addOp("RZ", self.qubitsToList([self.__id, phi]))
        operator = super().RZ(self.__id, phi)
        self.setOperation(operator)

    def CX(self, control):
        self.addOp("CX", self.qubitsToList([control, self.__id]))
        operator = super().CX(control, self.__id)
        self.setOperation(operator)

    def CNOT(self, control):
        self.CX(control)

    def CY(self, control):
        self.addOp("CY", self.qubitsToList([control, self.__id]))
        operator = super().CY(control, self.__id)
        self.setOperation(operator)

    def CZ(self, control):
        self.addOp("CZ", self.qubitsToList([control, self.__id]))
        operator = super().CZ(control, self.__id)
        self.setOperation(operator)

    def CV(self, control, adjoint = False):
        self.addOp("CV", self.qubitsToList([control, self.__id]))
        operator = super().CV(control, self.__id, adjoint)
        self.setOperation(operator)

    def CS(self, control, adjoint = False):
        self.addOp("CS", self.qubitsToList([control, self.__id]))
        operator = super().CS(control, self.__id, adjoint)
        self.setOperation(operator)

    def CT(self, control, adjoint = False):
        self.addOp("CT", self.qubitsToList([control, self.__id]))
        operator = super().CT(control, self.__id, adjoint)
        self.setOperation(operator)

    def CRX(self, control, theta):
        self.addOp("CRX", self.qubitsToList([control, self.__id, theta]))
        operator = super().CRX(control, self.__id, theta)
        self.setOperation(operator)

    def CRY(self, control, theta):
        self.addOp("CRY", self.qubitsToList([control, self.__id, theta]))
        operator = super().CRY(control, self.__id, theta)
        self.setOperation(operator)

    def CRZ(self, control, phi):
        self.addOp("CRZ", self.qubitsToList([control, self.__id, phi]))
        operator = super().CRZ(control, self.__id, phi)
        self.setOperation(operator)

    def CU(self, control, *argv):
        self.addOp("CU", self.qubitsToList([control, self.__id]))
        operator = super().CU(control, self.__id, argv)
        self.setOperation(operator)

    def CU3(self, control, theta, phi, _lambda):
        self.addOp("CU3", self.qubitsToList([control, self.__id, theta, phi, _lambda]))
        operator = super().CU3(control, self.__id, theta, phi, _lambda)
        self.setOperation(operator)

    def CU2(self, control, phi, _lambda):
        self.addOp("CU2", self.qubitsToList([control, self.__id, phi, _lambda]))
        operator = super().CU2(control, self.__id, phi, _lambda)
        self.setOperation(operator)

    def CU1(self, control, _lambda):
        self.addOp("CU1", self.qubitsToList([control, self.__id, _lambda]))
        operator = super().CU1(control, self.__id, _lambda)
        self.setOperation(operator)

    def CCX(self, control1, control2):
        self.addOp("CCX", self.qubitsToList([control1, control2, self.__id]))
        operator = super().CCX(control1, control2, self.__id)
        self.setOperation(operator)

    def Toffoli(self, control1, control2):
        self.CCX(control1, control2)

    def Controlled(self, name, control, target, adjoint = False):
        self.addOp(name, self.qubitsToList([control, self.__id]))
        functions = {'X': super().CX, 'Y': super().CY, 'Z': super().CZ}
        operator = functions[name](control[0], control[1], self.__id)
        self.setOperation(operator)

class QubitRegister(Qubit):
    def __init__(self, number = 3):
        self.__number = number
        if(self.getUsedQubits() < self.getQubitsNumber()):
            self.__reg = [Qubit() for i in range(1, number+1)]

    def __getitem__(self, key):
        return self.__reg[key-1]

    def getQubits(self):
        return self.__reg

    def X(self):
        for qubit in self.__reg:
            qubit.X()

    def Y(self):
        for qubit in self.__reg:
            qubit.Y()

    def Z(self):
        for qubit in self.__reg:
            qubit.Z()

    def V(self, target):
        for qubit in self.__reg:
            qubit.V()

    def S(self, target):
        for qubit in self.__reg:
            qubit.S()

    def T(self, target):
        for qubit in self.__reg:
            qubit.T()

    def H(self):
        for qubit in self.__reg:
            qubit.H()



#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

import sympy as sp
import cupy as cp
from cmath import *

from logicqubit.hilbert import *


class Gates(Hilbert):

    def __init__(self, number_of_qubits=1):
        self.__number_of_qubits = number_of_qubits

    def ID(self):
        M = Matrix([[1, 0], [0, 1]], self.getCuda())
        return M

    def P0(self):
        M = Matrix([[1, 0], [0, 0]], self.getCuda())  # |0><0|
        return M

    def P1(self):
        M = Matrix([[0, 0], [0, 1]], self.getCuda())  # |1><1|
        return M

    def L0(self):
        M = Matrix([[0, 1], [0, 0]], self.getCuda())  # |0><1|
        return M

    def L1(self):
        M = Matrix([[0, 0], [1, 0]], self.getCuda())  # |1><0|
        return M

    def X(self, target=1):
        M = Matrix([[0, 1], [1, 0]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def Y(self, target=1):
        M = Matrix([[0, -1j], [1j, 0]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def Z(self, target=1):
        M = Matrix([[1, 0], [0, -1]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def V(self, target=1):
        M = Matrix([[1, -1j], [-1j, 1]], self.getCuda()) * ((1j + 1) / 2)  # sqrt(X) ou sqrt(NOT)
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def S(self, target=1, adjoint=False):
        M = Matrix([[1, 0], [0, 1j]], self.getCuda())  # sqrt(Z)
        if (adjoint):
            M = M.adjoint()
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def T(self, target=1):
        M = Matrix([[1, 0], [0, (1 + 1j) / sqrt(2)]], self.getCuda())  # sqrt(S)
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def H(self, target=1):
        M = Matrix([[1, 1], [1, -1]], self.getCuda()) * (1 / sqrt(2))
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def U(self, target, *argv):  # U or theta, phi and _lambda
        if (len(argv) == 1):
            M = Matrix(argv[0][0], self.getCuda())
        else:
            theta = argv[0]
            phi = argv[1]
            _lambda = argv[2]
            M = Matrix(
                [[exp(-1j * (phi + _lambda) / 2) * cos(theta / 2), -exp(-1j * (phi - _lambda) / 2) * sin(theta / 2)],
                 [exp(-1j * (phi - _lambda) / 2) * sin(theta / 2), exp(1j * (phi + _lambda)) * cos(theta / 2)]],
                self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def U3(self, target, theta, phi, _lambda):
        M = Matrix([[cos(theta / 2), -exp(1j * _lambda) * sin(theta / 2)],
                    [exp(1j * phi) * sin(theta / 2), exp(1j * (phi + _lambda)) * cos(theta / 2)]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def U2(self, target, phi, _lambda):
        M = Matrix([[1, -exp(1j * _lambda)], [exp(1j * phi), exp(1j * (phi + _lambda))]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def U1(self, target, _lambda):
        M = Matrix([[1, 0], [0, exp(1j * _lambda)]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def RX(self, target, theta):
        M = Matrix([[cos(theta / 2), -1j * sin(theta / 2)],
                    [-1j * sin(theta / 2), cos(theta / 2)]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def RY(self, target, theta):
        M = Matrix([[cos(theta / 2), -sin(theta / 2)],
                    [sin(theta / 2), cos(theta / 2)]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def RZ(self, target, phi):
        M = Matrix([[exp(-1j * phi / 2), 0], [0, exp(1j * phi / 2)]], self.getCuda())
        list = self.getOrdListSimpleGate(target, M)
        operator = self.kronProduct(list)
        return operator

    def CX(self, control, target):
        M = Matrix([[0, 1], [1, 0]], self.getCuda())  # X
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CNOT(self, control, target):
        return self.CX(control, target)

    def CY(self, control, target):
        M = Matrix([[0, -1j], [1j, 0]], self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CZ(self, control, target):
        M = Matrix([[1, 0], [0, -1]], self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CV(self, control, target, adjoint=False):
        M = Matrix([[1, -1j], [-1j, 1]], self.getCuda()) * ((1j + 1) / 2)  # sqrt(X) ou sqrt(NOT)
        if (adjoint):
            M = M.adjoint()
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CS(self, control, target, adjoint=False):
        M = Matrix([[1, 0], [0, 1j]], self.getCuda())  # sqrt(Z)
        if (adjoint):
            M = M.adjoint()
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CT(self, control, target, adjoint=False):
        M = Matrix([[1, 0], [0, (1 + 1j) / sqrt(2)]], self.getCuda())  # sqrt(S)
        if (adjoint):
            M = M.adjoint()
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CRX(self, control, target, theta):
        M = Matrix([[cos(theta / 2), -1j * sin(theta / 2)],
                    [-1j * sin(theta / 2), cos(theta / 2)]], self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CRY(self, control, target, theta):
        M = Matrix([[cos(theta / 2), -sin(theta / 2)],
                    [sin(theta / 2), cos(theta / 2)]], self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CRZ(self, control, target, phi):
        M = Matrix([[exp(-1j * phi / 2), 0], [0, exp(1j * phi / 2)]], self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CU(self, control, target, *argv):  # U or theta, phi and _lambda
        if (len(argv) == 1):
            M = Matrix(argv[0][0], self.getCuda())
        else:
            theta = argv[0]
            phi = argv[1]
            _lambda = argv[2]
            M = Matrix(
                [[exp(-1j * (phi + _lambda) / 2) * cos(theta / 2), -exp(-1j * (phi - _lambda) / 2) * sin(theta / 2)],
                 [exp(1j * (phi - _lambda) / 2) * sin(theta / 2), exp(1j * (phi + _lambda)) * cos(theta / 2)]],
                self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CU3(self, control, target, theta, phi, _lambda):
        M = Matrix([[cos(theta / 2), -exp(1j * _lambda) * sin(theta / 2)],
                    [exp(1j * phi) * sin(theta / 2), exp(1j * (phi + _lambda)) * cos(theta / 2)]], self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CU2(self, control, target, phi, _lambda):
        M = Matrix([[1, -exp(1j * _lambda)], [exp(1j * phi), exp(1j * (phi + _lambda))]], self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CU1(self, control, target, _lambda):
        M = Matrix([[1, 0], [0, exp(1j * _lambda)]], self.getCuda())
        list1, list2 = self.getOrdListCtrlGate(control, target, M)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def CCX(self, control1, control2, target):
        Gate = Matrix([[0, 1], [1, 0]], self.getCuda()) - self.ID()
        list1, list2 = self.getOrdListCtrl2Gate(control1, control2, target, Gate)
        operator = self.kronProduct(list1) + self.kronProduct(list2)
        return operator

    def Fredkin(self, control, target1, target2):
        # Gate = Matrix([[0, 1], [1, 0]])-self.ID()
        # list1,list2 = self.getOrdListCtrl2Gate(control1, control2, target, Gate)
        # operator = self.kronProduct(list1) + self.kronProduct(list2)
        return None

    def SWAP(self):
        M = Matrix([[1, 0, 0, 0], [0, 0, 1, 0],
                    [0, 1, 0, 0], [0, 0, 0, 1]], self.getCuda())
        return M

    def Toffoli(self, control1, control2, target):
        return self.CCX(control1, control2, target)

    def getOrdListSimpleGate(self, target, Gate):
        list = []
        if (self.isFirstLeft()):
            plist = range(1, self.__number_of_qubits + 1)
        else:
            plist = reversed(range(1, self.__number_of_qubits + 1))
        for i in plist:
            if i == target:
                list.append(Gate)
            else:
                list.append(Matrix([[1, 0], [0, 1]], self.getCuda()))
        return list

    def getOrdListCtrlGate(self, control, target, Gate):
        list1 = []
        list2 = []
        if (self.isFirstLeft()):
            plist = range(1, self.__number_of_qubits + 1)
        else:
            plist = reversed(range(1, self.__number_of_qubits + 1))
        for i in plist:
            if i == control:
                list1.append(self.P0())
                list2.append(self.P1())
            elif i == target:
                list1.append(self.ID())
                list2.append(Gate)
            else:
                list1.append(self.ID())
                list2.append(self.ID())
        return list1, list2

    def getOrdListCtrl2Gate(self, control1, control2, target, Gate):
        list1 = []
        list2 = []
        if (self.isFirstLeft()):
            plist = range(1, self.__number_of_qubits + 1)
        else:
            plist = reversed(range(1, self.__number_of_qubits + 1))
        for i in plist:
            if i == control1 or i == control2:
                list1.append(self.ID())
                list2.append(self.P1())
            elif i == target:
                list1.append(self.ID())
                list2.append(Gate)
            else:
                list1.append(self.ID())
                list2.append(self.ID())
        return list1, list2



#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

class LogicQubitError(Exception):

    def __init__(self, message):
        super().__init__(message)
        self.message = message

    def __str__(self):
        return repr(self.message)


#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

from logicqubit.utils import *

from IPython.display import display, Math, Latex

# Zhegalkin polynomial
class Zhegalkin_Poly:
    def __init__(self):
        self.truth_table = []
        self.e = []
        self.p = []
        self.e.append(sp.Matrix([[1], [0]]))  # e0 = x+1
        self.e.append(sp.Matrix([[0], [1]]))  # e1 = x
        self.p.append(sp.Matrix([[1], [1]]))  # p0
        self.p.append(sp.Matrix([[0], [1]]))  # p1
        self.elist = []
        self.plist = []
        self.simplified_elist = []
        self.simplified_plist = []

    def kronProduct(self, list): # produto de Kronecker
        A = list[0]
        for M in list[1:]:
            A = TensorProduct(A, M)
        return A

    def addTable(self, table):
        self.truth_table.append(table)

    def SumPoly(self, poly1, poly2):
        poly = [0] * len(poly1)
        for i in range(len(poly)):
            poly[i] = int(poly1[i]) ^ int(poly2[i])  # p_ij * p_kl
        return poly

    def Compute(self):
        for i in range(len(self.truth_table)):
            self.elist.append([])
            self.plist.append([])
            for value in Utils.Text2List(self.truth_table[i]):
                t_elist = []
                t_plist = []
                for j in value:
                    t_elist.append(self.e[j])
                    t_plist.append(self.p[j])
                self.elist[i].append(self.kronProduct(t_elist))
                self.plist[i].append(self.kronProduct(t_plist))

        for i in range(len(self.truth_table)):
            _poly = self.plist[i][0]
            for poly in self.plist[i][1:]:
                _poly = self.SumPoly(_poly, poly)
            self.simplified_plist.append(_poly)
        return self.simplified_plist

    def ShowPolynomial(self, short = True):
        poly = []
        x = ["x_"+str(i+1) for i in range(len(self.truth_table[0][0]))]
        size_p = len(x)
        binlist = Utils.BinList(len(x))
        for p in self.simplified_plist:
            terms = ''
            for i in range(2**size_p):  # todos estados possíveis
                if(short):
                    if(p[i]==1):
                        if(len(terms) > 0):
                            terms += "\\oplus "
                        for j in range(len(x)):
                            terms += x[j]+"^"+str(binlist[i][j])
                else:
                    if (i == 0):
                        terms += str(p[i])
                    else:
                        terms += "\\oplus "+str(p[i])
                    for j in range(len(x)):
                        terms += x[j]+"^"+str(binlist[i][j])
            poly.append(terms)
            display(Math(terms))
        #expand = sp.expand((1 + x1) * (1 + x2))


#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

import sympy as sp
from sympy.physics.quantum import TensorProduct
from IPython.display import display, Math, Latex

from logicqubit.hilbert import *
from logicqubit.gates import *
from logicqubit.circuit import *
from logicqubit.utils import *
from logicqubit.zhegalkin import *

class Oracle:
    def __init__(self, input_qubits=[], type="truth_table"):
        self.__type = type
        self.__input_qubits = input_qubits
        self.__targets = []
        self.__truth_table = []

    def addInputQubits(self, values):
        self.__input_qubits = values

    def addTargets(self, values):
        self.__targets = values

    def addTable(self, target, values):
        self.__targets.append(target)
        self.__truth_table.append(values)

    def get(self):
        return self.__targets, self.__input_qubits, self.__truth_table



#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

import sympy as sp
from sympy.physics.quantum import TensorProduct
import numpy as np
import cupy as cp

from logicqubit.utils import *

class DecompositionPauliGates:

    def __init__(self, H):
        self.H = H  # hermitian matrix

    def sigma(self):
        ID = np.array([[1, 0], [0, 1]])
        X = np.array([[0, 1], [1, 0]])
        Y = np.array([[0, -1j], [1j, 0]])
        Z = np.array([[1, 0], [0, -1]])
        return [ID, X, Y, Z]

    def get_a(self):
        a = [[0]*4 for i in range(4)]
        for i, sigma_i in enumerate(self.sigma()):
            for j, sigma_j in enumerate(self.sigma()):
                a[i][j] = np.dot(np.kron(sigma_i, sigma_j), self.H).trace()
        return a


#!/usr/bin/python
# -*- coding: UTF-8 -*-

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

import sympy as sp
from sympy.physics.quantum import TensorProduct
import cupy as cp

class Utils:

    @staticmethod
    def onehot(i, value):
        if(i == value):
            return 1
        else:
            return 0

    @staticmethod
    def texfix(value, number, left=False):
        tex = sp.latex(value).replace(' \cdot ', '')
        tex = Utils.textSymbolfix(tex, number, left)
        return tex

    @staticmethod
    def textSymbolfix(value, number, left=False):
        text = value
        for i in range(1, number+1):
            if(left):
                text = text.replace(str(i) + 'a', 'a')
                text = text.replace(str(i) + 'b', 'b')
            else:
                text = text.replace(str(number+1-i) + 'a', 'a')
                text = text.replace(str(number+1-i) + 'b', 'b')
        return text

    @staticmethod
    def vec2tex(vector):
        tex = "\\begin{pmatrix}"+"{:g}".format(vector[0].item()).replace("+0j","")
        for value in vector[1:]:
            tex += ' \\\\ '+"{:g}".format(value.item()).replace("+0j","")
        tex += " \\end{pmatrix}"
        return tex

    @staticmethod
    def BinList(n):
        blist = []
        for i in iter(range(2 ** n)):
            b = bin(i)[2:].zfill(n)  # value in binary, ex: i=1, n=4 -> '0001'
            blist.append(b)
        return Utils.Text2List(blist)

    @staticmethod
    def Text2List(table):
        list = [int(i, base=2) for i in table]
        size = len(table[0])
        tmp = sorted(list, key=int, reverse=False)  # values in ascending order
        result = [[int(bin(j)[2:].zfill(size)[i]) for i in range(size)] for j in tmp]
        return result


#!/usr/bin/python
# encoding: utf-8

# Author Cleoner S. Pietralonga
# e-mail: cleonerp@gmail.com
# Apache License

class Circuit():

    def __init__(self):
        Circuit.__operations = []

    def addOp(self, operation, values):
        op = str(operation)+"("+str(values[0])
        for value in values[1:]:
            op+=","+str(value)
        op += ")"
        Circuit.__operations.append(op)

    def getOp(self):
        return Circuit.__operations

    def PrintOperations(self):
        print(self.__operations)


from setuptools import setup, find_packages
from os import path
from io import open
import logicqubit

# sudo pip3 install twine
# python setup.py sdist
# python setup.py bdist_wheel
# twine upload dist/*
# sudo python setup.py install

here = path.abspath(path.dirname(__file__))

with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='logicqubit-gpu',
    version=logicqubit.__version__,
    description='logicqubit-gpu is a simple library for quantum computing simulation with gpu acceleration.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/clnrp/logicqubit-gpu',
    author='Cleoner Pietralonga',
    author_email='cleonerp@gmail.com',

    classifiers=[
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        'Intended Audience :: Developers',
        'Topic :: Scientific/Engineering :: Physics',

        'License :: OSI Approved :: Apache Software License',

        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],

    packages=find_packages(exclude=['contrib', 'docs', 'tests']),
    install_requires=['sympy','numpy','cupy'],

)



from logicqubit import logic
from logicqubit import gates
from logicqubit import hilbert
from logicqubit import utils
from logicqubit import circuit
from logicqubit import zhegalkin
from logicqubit import oracle
from logicqubit import synthesis

__version__ = "1.5.2"

print("logicqubit-gpu version "+__version__)
